import pygame
import sys
import time
from pygame.locals import *
from pygame.color import *
import pymunk
pygame.init()
pygame.display.set_mode()
#-- Initialise the clock
clock = pygame.time.Clock()
#-- Initialise the physics engine
space = pymunk.Space()
space.gravity = (0.0,  0.0)
space.damping = 0.1# Adds friction to the ground for all objects
#-- Import from the ctf framework
import ai
import images
import gameobjects
import maps
import menu
import print_functions
import init_objects


#######  Collision functions #################
        
def collision_bullet_wall(arb, space, data):
     """Handle the collision between bullet and wall."""
     
     bullet = arb.shapes[0].parent
     game_objects_list.remove(bullet)
     space.remove(arb.shapes[0], arb.shapes[0].body)
     return False

def collision_bullet_box(arb, space, data):
    """Handle the collision between bullet and box."""
    
    bullet = arb.shapes[0].parent
    space.remove(arb.shapes[0], arb.shapes[0].body)
    if bullet in game_objects_list:
        game_objects_list.remove(bullet)
    box = arb.shapes[1].parent
    if box.destructable:
        if box.hitpoints > 1:
            box.hitpoints = box.hitpoints-1
        else:
            if box in game_objects_list:
                game_objects_list.remove(box)
            space.remove(arb.shapes[1], arb.shapes[1])
    return False

def collision_bullet_tank(arb, space, data):
    """Handle the collision between bullet and tank."""
    
    bullet = arb.shapes[0].parent
    tank = arb.shapes[1].parent
    if bullet in game_objects_list:
        game_objects_list.remove(bullet)
    space.remove(arb.shapes[0], arb.shapes[0].body)
    flag.is_on_tank = False
    tank.on_death()
    return False

            
def collision_bullet_bullet(arb, space, data):
    """Handle the collision between bullet and bullet."""
    
    bullet1= arb.shapes[1].parent
    bullet2 = arb.shapes[0].parent
    if bullet1 in game_objects_list:
        game_objects_list.remove(bullet1)
    space.remove(arb.shapes[0], arb.shapes[0].body)
    if bullet2 in game_objects_list:
        game_objects_list.remove(bullet2)
    space.remove(arb.shapes[1], arb.shapes[1].body)
    return False

def create_collision_handlers():
    """Creates the handling of collisions
       between different objects."""
    
    handler = space.add_collision_handler(1, 3)
    handler.pre_solve = collision_bullet_box
    handler = space.add_collision_handler(1, 10)
    handler.pre_solve = collision_bullet_wall
    handler = space.add_collision_handler(1, 2)
    handler.pre_solve = collision_bullet_tank
    handler = space.add_collision_handler(1, 1)
    handler.pre_solve = collision_bullet_bullet

def create_objects():
    """Creates objects in the game."""
    
    init_objects.create_borders(space,current_map)
    init_objects.create_background(current_map,background)
    init_objects.create_boxes(game_objects_list, current_map, space)
    init_objects.create_tanks(tanks_list,game_objects_list , \
                              current_map, space, multi, ai_list)
    init_objects.create_bases(game_objects_list , current_map)


############## Main loop ######################
def shoot_bullet(i):
    """Shoots a bullet from the tank
       and adds it to the game objects list."""
    
    bullet = tanks_list[i].shoot()
    if bullet != None:
        game_objects_list.append(bullet)
        

def keyhandler():
    """Handles key events for the game."""
    
    dic_keydown = {
            K_UP: tanks_list[0].accelerate,
            K_DOWN: tanks_list[0].decelerate,
            K_RIGHT: tanks_list[0].turn_right,
            K_LEFT: tanks_list[0].turn_left,
            K_ESCAPE: pygame.quit}

    dic_keyup = {
                 K_UP: tanks_list[0].stop_moving,
                 K_DOWN: tanks_list[0].stop_moving,
                 K_LEFT: tanks_list[0].stop_turning,
                 K_RIGHT: tanks_list[0].stop_turning }
    if multi:
        dic_keydown_multi = {
            K_w: tanks_list[1].accelerate,
            K_s: tanks_list[1].decelerate,
            K_d: tanks_list[1].turn_right,
            K_a: tanks_list[1].turn_left}

        dic_keyup_multi = {

                 K_w: tanks_list[1].stop_moving,
                 K_s: tanks_list[1].stop_moving,
                 K_d: tanks_list[1].stop_turning,
                 K_a: tanks_list[1].stop_turning}
        
        dic_keydown.update(dic_keydown_multi)
        dic_keyup.update(dic_keyup_multi)
    return dic_keydown, dic_keyup 
    
    
def tank_win(tank, max_round, max_time):
    """Handles the scenario when a tank wins the game."""
    
    player = 1
    flag.x, flag.y = flag_start
    flag.is_on_tank = False
    tank.flag = None
    tank.points = tank.points +1
    for i in tanks_list:
        i.stop_moving() # make sure no speed when respawn
        i.body.position = i.start_position
        print("Player",player,":",i.points)
        player = player+1
    print_functions.print_score_result(screen,tanks_list)
    print("============")
    
    if tank.points == 2 or max_round == 10  or max_time>1800*5: # 1800 == 1 min
        return False
    return True


def fog_of_war():
    """Implements the fog of war effect for the game."""
    
    tank_pixle_pos = 40
    tile_pixle_size = 100
    fog_pixle_radius = 90
    color_black = (255,255,255)
    color_white = (0,0,0)
    
    map_pixle_size = ((current_map.width+1)*tile_pixle_size,\
                      (current_map.height+1)*tile_pixle_size)
    x = tanks_list[0].body.position[0]*tank_pixle_pos
    y = tanks_list[0].body.position[1]*tank_pixle_pos
    surface1 = pygame.Surface(map_pixle_size)
    pygame.draw.circle(surface1, color_black, (x,y), \
                       fog_pixle_radius,pygame.SRCALPHA)

    # add the extra fog vision in case of multiplayer 
    if multi:
        x_multi = tanks_list[1].body.position[0]*tank_pixle_pos
        y_multi = tanks_list[1].body.position[1]*tank_pixle_pos
        pygame.draw.circle(surface1, color_black,(x_multi,y_multi),
                           fog_pixle_radius,pygame.SRCALPHA)
        
    pygame.draw.rect(surface1, color_white,(map_pixle_size[0], \
                                            map_pixle_size[1],0,0))
    surface1.set_colorkey(color_black)  
    screen.blit(surface1, (0,0))
    
def main_loop():
    """The main loop of the game where events are handled,
       game objects are updated and displayed on the screen."""
    
    
    # init key inputs!
    dic_keydown, dic_keyup = keyhandler()
    
    running = True
    skip_update = 0

    # Create the tank, images.tanks contains the image representing the tank
    count_rounds = 1
    count_time = 0
    
    while running:
        count_time = count_time +1

        for event in pygame.event.get():
            if event.type == KEYDOWN:
                if event.key in dic_keydown:
                    dic_keydown[event.key]()
                elif event.key == K_RETURN:
                    shoot_bullet(0)
                elif event.key == K_SPACE:
                    shoot_bullet(1)
            
            elif event.type == KEYUP:
                if event.key in dic_keyup:
                    dic_keyup[event.key]()
            
#-- Update physics

        if skip_update == 0:

            for obj in game_objects_list:
                obj.update()
            skip_update = 2
        else:
            skip_update -= 1
            
        space.step(1 / FRAMERATE)

    #<INSERT DISPLAY BACKGROUND>

# Display the background on the screen
        screen.blit(background, (0, 0))

    #<INSERT DISPLAY OBJECTS>
        
    # Update the display of the game objects on the screen
        for i in tanks_list:
            if i.has_won():
                count_rounds = count_rounds +1
                running = tank_win(i,count_rounds, count_time)
            i.post_update(flag)
            
                
        for i in ai_list:
            i.decide()
            
        for obj in game_objects_list:
            obj.update_screen(screen)
            obj.update()

        if FOG:
            fog_of_war()
            
        if game_mode == 1:
            print_functions.print_stats_score(screen,\
                                              tanks_list, current_map)
            
        elif game_mode == 2:
            print_functions.print_stats_time(screen, \
                                             count_time, current_map)
            
        elif game_mode == 3:
            print_functions.print_stats_round(screen,\
                                              count_rounds , current_map)
      
    #   Redisplay the entire screen (see double buffer technique)  
        pygame.display.flip()

    #   Control the game framerate
        clock.tick(FRAMERATE)

    #   Redisplay the entire screen (see double buffer technique)
        pygame.display.flip()

    #   Control the game framerate
        clock.tick(FRAMERATE)

# main
try:
    if str(sys.argv[1]) == "--singleplayer":
        multi = False
    if str(sys.argv[1]) == "--hot-multiplayer":
        multi = True
        FOG = False
        current_map = maps.map0
        game_mode = 1

except IndexError:
    main_tuple = menu.main_menu()
    multi = main_tuple[0]
    current_map = main_tuple[1]
    FOG = main_tuple[2]
    game_mode = main_tuple[3]

#----- Initialisation -----#

#-- Initialise the display

#-- Constants
FRAMERATE = 100

#-- Variables

#   Define the current level

flag_start  = current_map.flag_position[0], current_map.flag_position[1]
#   List of all game objects

game_objects_list   = []
tanks_list          = []
ai_list             = []

#-- Resize the screen to the size of the current level
screen = pygame.display.set_mode((current_map.rect().size[0]+100,\
                                  current_map.rect().size[1]))

#<INSERT GENERATE BACKGROUND>

#-- Generate the background
background = pygame.Surface(screen.get_size())

flag = init_objects.create_flag(game_objects_list,flag_start)
create_objects()
create_collision_handlers()
main_loop()
