import gameobjects
from pygame.locals import *
from pygame.color import *
import pymunk
import images
import ai
def create_flag(game_objects_list,flag_start):
    """Create a flag object and add it to the list of game objects."""

    flag = gameobjects.Flag(flag_start[0], flag_start[1])
    game_objects_list.append(flag)
    flag.is_on_tank   = False
    flag.x, flag.y = flag_start
    return flag

def create_borders(space,current_map):
    """ Create borders for the game map. """
    
    static_body = space.static_body

    static_lines = [
            pymunk.Segment(static_body,
                           (-0.5, current_map.height), (-0.5, 0), 0.5
                            ), # left
            pymunk.Segment(static_body, (0, current_map.height+0.5),
                           (current_map.width, current_map.height+0.5),
                            0.5), # lower
            pymunk.Segment(static_body,
                           (current_map.width+0.5, 0), (current_map.width+0.5,
                            current_map.height),
                           0.5), # right
            pymunk.Segment(static_body, (0, -0.5), (current_map.width, -0.5),
                            0.5) # upper
        ]
    static_lines[0].collision_type = 10
    static_lines[1].collision_type = 10
    static_lines[2].collision_type = 10
    static_lines[3].collision_type = 10
    space.add(*static_lines)
    
    
def create_background(current_map,background):
    """Create a background image for the game map."""
    for x in range(0, current_map.width):
        for y in range(0,  current_map.height):
            background.blit(images.grass,(x*images.TILE_SIZE, \
                                          y*images.TILE_SIZE))
            
            
def create_boxes(game_objects_list , current_map, space):
    """Create boxes for the game map."""
    
    for x in range(0, current_map.width):
        for y in range(0,  current_map.height):
        # Get the type of boxes
            box_type  = current_map.boxAt(x, y)
            if(box_type != 0):
                box = gameobjects.get_box_with_type(x, y, box_type, space)
                game_objects_list.append(box)

def create_tanks(tanks_list,game_objects_list,current_map,space,multi,ai_list):
    """  create and manage tanks in the game. """
    
    for i in range(0, len(current_map.start_positions)):
    # Get the starting position of the tank "i"
        pos = current_map.start_positions[i]
    # Create the tank, images.tanks contains the image representing the tank
        tank = gameobjects.Tank(pos[0], pos[1], pos[2], images.tanks[i], space)
    # Add the tank to the list of objects to display
        game_objects_list.append(tank)
    # Add the tank to the list of tanks
        tanks_list.append(tank)
        if not multi:
            for i in range(len(tanks_list)-1):
                ai_list.append(ai.Ai(tanks_list[i+1], game_objects_list,\
                                     tanks_list, space, current_map))
        else:
            for i in range(1,len(tanks_list)-1):
                ai_list.append(ai.Ai(tanks_list[i+1], game_objects_list,\
                                     tanks_list, space, current_map))   
               

def create_bases(game_objects_list,current_map):
    """ create all needed bases in the game. """
    
    for i in range(0, len(current_map.start_positions)):
        pos = current_map.start_positions[i]
        base = gameobjects.GameVisibleObject(pos[0],pos[1],images.bases[i])
        game_objects_list.append(base)
